package javax.swing;

import java.awt.event.WindowEvent;
import java.awt.Insets;
import java.awt.Image;
import sun.reflect.misc.ReflectUtil;
import sun.awt.AppContext;
import java.awt.event.WindowListener;
import java.awt.HeadlessException;
import java.awt.Frame;
import java.awt.event.ActionEvent;
import sun.swing.UIAction;
import java.awt.event.KeyEvent;
import java.awt.KeyboardFocusManager;
import javax.accessibility.AccessibleStateSet;
import javax.accessibility.AccessibleComponent;
import javax.accessibility.AccessibleContext;
import javax.accessibility.Accessible;
import java.lang.reflect.InvocationTargetException;
import java.awt.EventQueue;
import java.awt.Graphics;
import javax.swing.text.View;
import sun.swing.SwingUtilities2;
import java.awt.FontMetrics;
import java.awt.IllegalComponentStateException;
import java.applet.Applet;
import sun.awt.AWTAccessor;
import javax.swing.event.MenuDragMouseEvent;
import java.awt.event.MouseWheelEvent;
import java.awt.event.MouseEvent;
import java.awt.Point;
import java.awt.Container;
import java.awt.Window;
import java.awt.Rectangle;
import java.awt.dnd.DropTarget;
import java.awt.GraphicsEnvironment;
import javax.swing.plaf.UIResource;
import java.awt.Component;
import java.security.AccessController;
import sun.security.action.GetPropertyAction;

public class SwingUtilities implements SwingConstants
{
    private static boolean canAccessEventQueue;
    private static boolean eventQueueTested;
    private static boolean suppressDropSupport;
    private static boolean checkedSuppressDropSupport;
    private static final Object sharedOwnerFrameKey;
    
    private static boolean getSuppressDropTarget() {
        if (!SwingUtilities.checkedSuppressDropSupport) {
            SwingUtilities.suppressDropSupport = (boolean)Boolean.valueOf((String)AccessController.doPrivileged(new GetPropertyAction("suppressSwingDropSupport")));
            SwingUtilities.checkedSuppressDropSupport = true;
        }
        return SwingUtilities.suppressDropSupport;
    }
    
    static void installSwingDropTargetAsNecessary(Component component, TransferHandler transferHandler) {
        if (!getSuppressDropTarget()) {
            DropTarget dropTarget = component.getDropTarget();
            if (dropTarget == null || dropTarget instanceof UIResource) {
                if (transferHandler == null) {
                    component.setDropTarget(null);
                }
                else if (!GraphicsEnvironment.isHeadless()) {
                    component.setDropTarget(new TransferHandler.SwingDropTarget(component));
                }
            }
        }
    }
    
    public static final boolean isRectangleContainingRectangle(Rectangle rectangle, Rectangle rectangle2) {
        return rectangle2.x >= rectangle.x && rectangle2.x + rectangle2.width <= rectangle.x + rectangle.width && rectangle2.y >= rectangle.y && rectangle2.y + rectangle2.height <= rectangle.y + rectangle.height;
    }
    
    public static Rectangle getLocalBounds(Component component) {
        Rectangle rectangle2;
        Rectangle rectangle = rectangle2 = new Rectangle(component.getBounds());
        int n = 0;
        rectangle.y = n;
        rectangle2.x = n;
        return rectangle;
    }
    
    public static Window getWindowAncestor(Component component) {
        Container container = component.getParent();
        while (container != null) {
            if ((container instanceof Window)) {
                return (Window)container;
            }
            container = container.getParent();
        }
        return null;
    }
    
    static Point convertScreenLocationToParent(Container parent, int x, int y) {
        while (parent != null) {
            if ((parent instanceof Window)) {
                Point o = new Point(x, y);
                convertPointFromScreen(o, parent);
                return o;
            }
            parent = parent.getParent();
        }
        throw new Error("convertScreenLocationToParent: no window ancestor");
    }
    
    public static Point convertPoint(Component windowAncestor, Point p2, Component windowAncestor2) {
        if (windowAncestor == null && windowAncestor2 == null) {
            return p2;
        }
        if (windowAncestor == null) {
            windowAncestor = getWindowAncestor(windowAncestor2);
            if (windowAncestor == null) {
                throw new Error("Source component not connected to component tree hierarchy");
            }
        }
        Point o = new Point(p2);
        convertPointToScreen(o, windowAncestor);
        if (windowAncestor2 == null) {
            windowAncestor2 = getWindowAncestor(windowAncestor);
            if (windowAncestor2 == null) {
                throw new Error("Destination component not connected to component tree hierarchy");
            }
        }
        convertPointFromScreen(o, windowAncestor2);
        return o;
    }
    
    public static Point convertPoint(Component component, int x, int y, Component component2) {
        return convertPoint(component, new Point(x, y), component2);
    }
    
    public static Rectangle convertRectangle(Component component, Rectangle rectangle, Component component2) {
        Point convertPoint = convertPoint(component, new Point(rectangle.x, rectangle.y), component2);
        return new Rectangle(convertPoint.x, convertPoint.y, rectangle.width, rectangle.height);
    }
    
    public static Container getAncestorOfClass(Class<?> clazz, Component component) {
        if (component == null || clazz == null) {
            return null;
        }
        Container container;
        for (container = component.getParent(); container != null && !clazz.isInstance(container); container = container.getParent()) {}
        return container;
    }
    
    public static Container getAncestorNamed(String s, Component component) {
        if (component == null || s == null) {
            return null;
        }
        Container container;
        for (container = component.getParent(); container != null && !s.equals(container.getName()); container = container.getParent()) {}
        return container;
    }
    
    public static Component getDeepestComponentAt(Component component, int x, int y) {
        if (!component.contains(x, y)) {
            return null;
        }
        if ((component instanceof Container)) {
            Component[] components;
            for (Component component2 : components = ((Container)component).getComponents()) {
                if (component2 != null && component2.isVisible()) {
                    Point location = component2.getLocation();
                    Component component3;
                    if ((component2 instanceof Container)) {
                        component3 = getDeepestComponentAt(component2, x - location.x, y - location.y);
                    }
                    else {
                        component3 = component2.getComponentAt(x - location.x, y - location.y);
                    }
                    if (component3 != null && component3.isVisible()) {
                        return component3;
                    }
                }
            }
            return component;
        }
        return component;
    }
    
    public static MouseEvent convertMouseEvent(Component component, MouseEvent mouseEvent, Component source) {
        Point convertPoint = convertPoint(component, new Point(mouseEvent.getX(), mouseEvent.getY()), source);
        if (source != null) {
        }
        else {
            source = component;
        }
        Object o;
        if ((mouseEvent instanceof MouseWheelEvent)) {
            MouseWheelEvent mouseWheelEvent = (MouseWheelEvent)mouseEvent;
            o = new MouseWheelEvent(source, mouseWheelEvent.getID(), mouseWheelEvent.getWhen(), mouseWheelEvent.getModifiers() | mouseWheelEvent.getModifiersEx(), convertPoint.x, convertPoint.y, mouseWheelEvent.getXOnScreen(), mouseWheelEvent.getYOnScreen(), mouseWheelEvent.getClickCount(), mouseWheelEvent.isPopupTrigger(), mouseWheelEvent.getScrollType(), mouseWheelEvent.getScrollAmount(), mouseWheelEvent.getWheelRotation());
        }
        else if ((mouseEvent instanceof MenuDragMouseEvent)) {
            MenuDragMouseEvent menuDragMouseEvent = (MenuDragMouseEvent)mouseEvent;
            o = new MenuDragMouseEvent(source, menuDragMouseEvent.getID(), menuDragMouseEvent.getWhen(), menuDragMouseEvent.getModifiers() | menuDragMouseEvent.getModifiersEx(), convertPoint.x, convertPoint.y, menuDragMouseEvent.getXOnScreen(), menuDragMouseEvent.getYOnScreen(), menuDragMouseEvent.getClickCount(), menuDragMouseEvent.isPopupTrigger(), menuDragMouseEvent.getPath(), menuDragMouseEvent.getMenuSelectionManager());
        }
        else {
            o = new MouseEvent(source, mouseEvent.getID(), mouseEvent.getWhen(), mouseEvent.getModifiers() | mouseEvent.getModifiersEx(), convertPoint.x, convertPoint.y, mouseEvent.getXOnScreen(), mouseEvent.getYOnScreen(), mouseEvent.getClickCount(), mouseEvent.isPopupTrigger(), mouseEvent.getButton());
            AWTAccessor.MouseEventAccessor mouseEventAccessor = AWTAccessor.getMouseEventAccessor();
            mouseEventAccessor.setCausedByTouchEvent(o, mouseEventAccessor.isCausedByTouchEvent(mouseEvent));
        }
        return o;
    }
    
    public static void convertPointToScreen(Point point, Component parent) {
        do {
            int n;
            int n2;
            Label_0084: {
                if ((parent instanceof JComponent)) {
                    n = parent.getX();
                    n2 = parent.getY();
                }
                else {
                    if (!(parent instanceof Applet) && !(parent instanceof Window)) {
                        n = parent.getX();
                        n2 = parent.getY();
                        break Label_0084;
                    }
                    try {
                        Point locationOnScreen = parent.getLocationOnScreen();
                        n = locationOnScreen.x;
                        n2 = locationOnScreen.y;
                    }
                    catch (IllegalComponentStateException ex) {
                        n = parent.getX();
                        n2 = parent.getY();
                    }
                }
            }
            point.x += n;
            point.y += n2;
            if (!(parent instanceof Window)) {
                if ((parent instanceof Applet)) {
                    break;
                }
                parent = parent.getParent();
            }
            else {
                break;
            }
        } while (parent != null);
    }
    
    public static void convertPointFromScreen(Point point, Component parent) {
        do {
            int n;
            int n2;
            Label_0084: {
                if ((parent instanceof JComponent)) {
                    n = parent.getX();
                    n2 = parent.getY();
                }
                else {
                    if (!(parent instanceof Applet) && !(parent instanceof Window)) {
                        n = parent.getX();
                        n2 = parent.getY();
                        break Label_0084;
                    }
                    try {
                        Point locationOnScreen = parent.getLocationOnScreen();
                        n = locationOnScreen.x;
                        n2 = locationOnScreen.y;
                    }
                    catch (IllegalComponentStateException ex) {
                        n = parent.getX();
                        n2 = parent.getY();
                    }
                }
            }
            point.x -= n;
            point.y -= n2;
            if (!(parent instanceof Window)) {
                if ((parent instanceof Applet)) {
                    break;
                }
                parent = parent.getParent();
            }
            else {
                break;
            }
        } while (parent != null);
    }
    
    public static Window windowForComponent(Component component) {
        return getWindowAncestor(component);
    }
    
    public static boolean isDescendingFrom(Component component, Component component2) {
        if (component == component2) {
            return true;
        }
        Container container = component.getParent();
        while (container != null) {
            if (container == component2) {
                return true;
            }
            container = container.getParent();
        }
        return false;
    }
    
    public static Rectangle computeIntersection(int n, int n2, int n3, int n4, Rectangle rectangle) {
        int x = (n > rectangle.x) ? n : rectangle.x;
        int n5 = (n + n3 < rectangle.x + rectangle.width) ? (n + n3) : (rectangle.x + rectangle.width);
        int y = (n2 > rectangle.y) ? n2 : rectangle.y;
        int n6 = (n2 + n4 < rectangle.y + rectangle.height) ? (n2 + n4) : (rectangle.y + rectangle.height);
        rectangle.x = x;
        rectangle.y = y;
        rectangle.width = n5 - x;
        rectangle.height = n6 - y;
        if (rectangle.width < 0 || rectangle.height < 0) {
            int n7 = 0;
            rectangle.height = n7;
            rectangle.width = n7;
            rectangle.y = n7;
            rectangle.x = n7;
            return rectangle;
        }
        return rectangle;
    }
    
    public static Rectangle computeUnion(int n, int n2, int n3, int n4, Rectangle rectangle) {
        int x = (n < rectangle.x) ? n : rectangle.x;
        int n5 = (n + n3 > rectangle.x + rectangle.width) ? (n + n3) : (rectangle.x + rectangle.width);
        int y = (n2 < rectangle.y) ? n2 : rectangle.y;
        int n6 = (n2 + n4 > rectangle.y + rectangle.height) ? (n2 + n4) : (rectangle.y + rectangle.height);
        rectangle.x = x;
        rectangle.y = y;
        rectangle.width = n5 - x;
        rectangle.height = n6 - y;
        return rectangle;
    }
    
    public static Rectangle[] computeDifference(Rectangle rectangle, Rectangle r) {
        if (r == null || !rectangle.intersects(r) || isRectangleContainingRectangle(r, rectangle)) {
            return new Rectangle[0];
        }
        Rectangle rectangle2 = new Rectangle();
        Rectangle rectangle3 = null;
        Object o = null;
        Object o2 = null;
        Object o3 = null;
        int n = 0;
        if (isRectangleContainingRectangle(rectangle, r)) {
            rectangle2.x = rectangle.x;
            rectangle2.y = rectangle.y;
            rectangle2.width = r.x - rectangle.x;
            rectangle2.height = rectangle.height;
            if (rectangle2.width > 0 && rectangle2.height > 0) {
                rectangle3 = new Rectangle(rectangle2);
                ++n;
            }
            rectangle2.x = r.x;
            rectangle2.y = rectangle.y;
            rectangle2.width = r.width;
            rectangle2.height = r.y - rectangle.y;
            if (rectangle2.width > 0 && rectangle2.height > 0) {
                o = new Rectangle(rectangle2);
                ++n;
            }
            rectangle2.x = r.x;
            rectangle2.y = r.y + r.height;
            rectangle2.width = r.width;
            rectangle2.height = rectangle.y + rectangle.height - (r.y + r.height);
            if (rectangle2.width > 0 && rectangle2.height > 0) {
                o2 = new Rectangle(rectangle2);
                ++n;
            }
            rectangle2.x = r.x + r.width;
            rectangle2.y = rectangle.y;
            rectangle2.width = rectangle.x + rectangle.width - (r.x + r.width);
            rectangle2.height = rectangle.height;
            if (rectangle2.width > 0 && rectangle2.height > 0) {
                o3 = new Rectangle(rectangle2);
                ++n;
            }
        }
        else if (r.x <= rectangle.x && r.y <= rectangle.y) {
            if (r.x + r.width > rectangle.x + rectangle.width) {
                rectangle2.x = rectangle.x;
                rectangle2.y = r.y + r.height;
                rectangle2.width = rectangle.width;
                rectangle2.height = rectangle.y + rectangle.height - (r.y + r.height);
                if (rectangle2.width > 0 && rectangle2.height > 0) {
                    rectangle3 = rectangle2;
                    ++n;
                }
            }
            else if (r.y + r.height > rectangle.y + rectangle.height) {
                rectangle2.setBounds(r.x + r.width, rectangle.y, rectangle.x + rectangle.width - (r.x + r.width), rectangle.height);
                if (rectangle2.width > 0 && rectangle2.height > 0) {
                    rectangle3 = rectangle2;
                    ++n;
                }
            }
            else {
                rectangle2.setBounds(r.x + r.width, rectangle.y, rectangle.x + rectangle.width - (r.x + r.width), r.y + r.height - rectangle.y);
                if (rectangle2.width > 0 && rectangle2.height > 0) {
                    rectangle3 = new Rectangle(rectangle2);
                    ++n;
                }
                rectangle2.setBounds(rectangle.x, r.y + r.height, rectangle.width, rectangle.y + rectangle.height - (r.y + r.height));
                if (rectangle2.width > 0 && rectangle2.height > 0) {
                    o = new Rectangle(rectangle2);
                    ++n;
                }
            }
        }
        else if (r.x <= rectangle.x && r.y + r.height >= rectangle.y + rectangle.height) {
            if (r.x + r.width > rectangle.x + rectangle.width) {
                rectangle2.setBounds(rectangle.x, rectangle.y, rectangle.width, r.y - rectangle.y);
                if (rectangle2.width > 0 && rectangle2.height > 0) {
                    rectangle3 = rectangle2;
                    ++n;
                }
            }
            else {
                rectangle2.setBounds(rectangle.x, rectangle.y, rectangle.width, r.y - rectangle.y);
                if (rectangle2.width > 0 && rectangle2.height > 0) {
                    rectangle3 = new Rectangle(rectangle2);
                    ++n;
                }
                rectangle2.setBounds(r.x + r.width, r.y, rectangle.x + rectangle.width - (r.x + r.width), rectangle.y + rectangle.height - r.y);
                if (rectangle2.width > 0 && rectangle2.height > 0) {
                    o = new Rectangle(rectangle2);
                    ++n;
                }
            }
        }
        else if (r.x <= rectangle.x) {
            if (r.x + r.width >= rectangle.x + rectangle.width) {
                rectangle2.setBounds(rectangle.x, rectangle.y, rectangle.width, r.y - rectangle.y);
                if (rectangle2.width > 0 && rectangle2.height > 0) {
                    rectangle3 = new Rectangle(rectangle2);
                    ++n;
                }
                rectangle2.setBounds(rectangle.x, r.y + r.height, rectangle.width, rectangle.y + rectangle.height - (r.y + r.height));
                if (rectangle2.width > 0 && rectangle2.height > 0) {
                    o = new Rectangle(rectangle2);
                    ++n;
                }
            }
            else {
                rectangle2.setBounds(rectangle.x, rectangle.y, rectangle.width, r.y - rectangle.y);
                if (rectangle2.width > 0 && rectangle2.height > 0) {
                    rectangle3 = new Rectangle(rectangle2);
                    ++n;
                }
                rectangle2.setBounds(r.x + r.width, r.y, rectangle.x + rectangle.width - (r.x + r.width), r.height);
                if (rectangle2.width > 0 && rectangle2.height > 0) {
                    o = new Rectangle(rectangle2);
                    ++n;
                }
                rectangle2.setBounds(rectangle.x, r.y + r.height, rectangle.width, rectangle.y + rectangle.height - (r.y + r.height));
                if (rectangle2.width > 0 && rectangle2.height > 0) {
                    o2 = new Rectangle(rectangle2);
                    ++n;
                }
            }
        }
        else if (r.x <= rectangle.x + rectangle.width && r.x + r.width > rectangle.x + rectangle.width) {
            if (r.y <= rectangle.y && r.y + r.height > rectangle.y + rectangle.height) {
                rectangle2.setBounds(rectangle.x, rectangle.y, r.x - rectangle.x, rectangle.height);
                if (rectangle2.width > 0 && rectangle2.height > 0) {
                    rectangle3 = rectangle2;
                    ++n;
                }
            }
            else if (r.y <= rectangle.y) {
                rectangle2.setBounds(rectangle.x, rectangle.y, r.x - rectangle.x, r.y + r.height - rectangle.y);
                if (rectangle2.width > 0 && rectangle2.height > 0) {
                    rectangle3 = new Rectangle(rectangle2);
                    ++n;
                }
                rectangle2.setBounds(rectangle.x, r.y + r.height, rectangle.width, rectangle.y + rectangle.height - (r.y + r.height));
                if (rectangle2.width > 0 && rectangle2.height > 0) {
                    o = new Rectangle(rectangle2);
                    ++n;
                }
            }
            else if (r.y + r.height > rectangle.y + rectangle.height) {
                rectangle2.setBounds(rectangle.x, rectangle.y, rectangle.width, r.y - rectangle.y);
                if (rectangle2.width > 0 && rectangle2.height > 0) {
                    rectangle3 = new Rectangle(rectangle2);
                    ++n;
                }
                rectangle2.setBounds(rectangle.x, r.y, r.x - rectangle.x, rectangle.y + rectangle.height - r.y);
                if (rectangle2.width > 0 && rectangle2.height > 0) {
                    o = new Rectangle(rectangle2);
                    ++n;
                }
            }
            else {
                rectangle2.setBounds(rectangle.x, rectangle.y, rectangle.width, r.y - rectangle.y);
                if (rectangle2.width > 0 && rectangle2.height > 0) {
                    rectangle3 = new Rectangle(rectangle2);
                    ++n;
                }
                rectangle2.setBounds(rectangle.x, r.y, r.x - rectangle.x, r.height);
                if (rectangle2.width > 0 && rectangle2.height > 0) {
                    o = new Rectangle(rectangle2);
                    ++n;
                }
                rectangle2.setBounds(rectangle.x, r.y + r.height, rectangle.width, rectangle.y + rectangle.height - (r.y + r.height));
                if (rectangle2.width > 0 && rectangle2.height > 0) {
                    o2 = new Rectangle(rectangle2);
                    ++n;
                }
            }
        }
        else if (r.x >= rectangle.x && r.x + r.width <= rectangle.x + rectangle.width) {
            if (r.y <= rectangle.y && r.y + r.height > rectangle.y + rectangle.height) {
                rectangle2.setBounds(rectangle.x, rectangle.y, r.x - rectangle.x, rectangle.height);
                if (rectangle2.width > 0 && rectangle2.height > 0) {
                    rectangle3 = new Rectangle(rectangle2);
                    ++n;
                }
                rectangle2.setBounds(r.x + r.width, rectangle.y, rectangle.x + rectangle.width - (r.x + r.width), rectangle.height);
                if (rectangle2.width > 0 && rectangle2.height > 0) {
                    o = new Rectangle(rectangle2);
                    ++n;
                }
            }
            else if (r.y <= rectangle.y) {
                rectangle2.setBounds(rectangle.x, rectangle.y, r.x - rectangle.x, rectangle.height);
                if (rectangle2.width > 0 && rectangle2.height > 0) {
                    rectangle3 = new Rectangle(rectangle2);
                    ++n;
                }
                rectangle2.setBounds(r.x, r.y + r.height, r.width, rectangle.y + rectangle.height - (r.y + r.height));
                if (rectangle2.width > 0 && rectangle2.height > 0) {
                    o = new Rectangle(rectangle2);
                    ++n;
                }
                rectangle2.setBounds(r.x + r.width, rectangle.y, rectangle.x + rectangle.width - (r.x + r.width), rectangle.height);
                if (rectangle2.width > 0 && rectangle2.height > 0) {
                    o2 = new Rectangle(rectangle2);
                    ++n;
                }
            }
            else {
                rectangle2.setBounds(rectangle.x, rectangle.y, r.x - rectangle.x, rectangle.height);
                if (rectangle2.width > 0 && rectangle2.height > 0) {
                    rectangle3 = new Rectangle(rectangle2);
                    ++n;
                }
                rectangle2.setBounds(r.x, rectangle.y, r.width, r.y - rectangle.y);
                if (rectangle2.width > 0 && rectangle2.height > 0) {
                    o = new Rectangle(rectangle2);
                    ++n;
                }
                rectangle2.setBounds(r.x + r.width, rectangle.y, rectangle.x + rectangle.width - (r.x + r.width), rectangle.height);
                if (rectangle2.width > 0 && rectangle2.height > 0) {
                    o2 = new Rectangle(rectangle2);
                    ++n;
                }
            }
        }
        Rectangle[] array = new Rectangle[n];
        int n2 = 0;
        if (rectangle3 != null) {
            array[n2++] = rectangle3;
        }
        if (o != null) {
            array[n2++] = o;
        }
        if (o2 != null) {
            array[n2++] = o2;
        }
        if (o3 != null) {
            array[n2++] = o3;
            return array;
        }
        return array;
    }
    
    public static boolean isLeftMouseButton(MouseEvent mouseEvent) {
        return (mouseEvent.getModifiersEx() & 0x400) != 0x0 || mouseEvent.getButton() == 1;
    }
    
    public static boolean isMiddleMouseButton(MouseEvent mouseEvent) {
        return (mouseEvent.getModifiersEx() & 0x800) != 0x0 || mouseEvent.getButton() == 2;
    }
    
    public static boolean isRightMouseButton(MouseEvent mouseEvent) {
        return (mouseEvent.getModifiersEx() & 0x1000) != 0x0 || mouseEvent.getButton() == 3;
    }
    
    public static int computeStringWidth(FontMetrics fm, String string) {
        return SwingUtilities2.stringWidth(null, fm, string);
    }
    
    public static String layoutCompoundLabel(JComponent component, FontMetrics fontMetrics, String s, Icon icon, int n, int n2, int n3, int n4, Rectangle rectangle, Rectangle rectangle2, Rectangle rectangle3, int n5) {
        boolean n6 = true;
        if (component != null && !component.getComponentOrientation().isLeftToRight()) {
            n6 = false;
        }
        switch (n2) {
            case 10: {
                n2 = (n6 ? 2 : 4);
                break;
            }
            case 11: {
                n2 = (n6 ? 4 : 2);
                break;
            }
        }
        switch (n4) {
            case 10: {
                n4 = (n6 ? 2 : 4);
                break;
            }
            case 11: {
                n4 = (n6 ? 4 : 2);
                break;
            }
        }
        return layoutCompoundLabelImpl(component, fontMetrics, s, icon, n, n2, n3, n4, rectangle, rectangle2, rectangle3, n5);
    }
    
    public static String layoutCompoundLabel(FontMetrics fontMetrics, String s, Icon icon, int n, int n2, int n3, int n4, Rectangle rectangle, Rectangle rectangle2, Rectangle rectangle3, int n5) {
        return layoutCompoundLabelImpl(null, fontMetrics, s, icon, n, n2, n3, n4, rectangle, rectangle2, rectangle3, n5);
    }
    
    private static String layoutCompoundLabelImpl(JComponent component, FontMetrics fontMetrics, String clipString, Icon icon, int n, int n2, int n3, int n4, Rectangle rectangle, Rectangle rectangle2, Rectangle rectangle3, int n5) {
        if (icon != null) {
            rectangle2.width = icon.getIconWidth();
            rectangle2.height = icon.getIconHeight();
        }
        else {
            int n6 = 0;
            rectangle2.height = n6;
            rectangle2.width = n6;
        }
        int n7 = clipString == null || clipString.equals("");
        int leftSideBearing = 0;
        int n8 = 0;
        int n9;
        if (!(n7 == 0)) {
            int n10 = 0;
            rectangle3.height = n10;
            rectangle3.width = n10;
            clipString = "";
            n9 = 0;
        }
        else {
            n9 = ((icon == null) ? 0 : n5);
            int width;
            if (n4 == 0) {
                width = rectangle.width;
            }
            else {
                width = rectangle.width - (rectangle2.width + n9);
            }
            View view = (component != null) ? ((View)component.getClientProperty("html")) : null;
            if (view != null) {
                rectangle3.width = Math.min(width, (int)view.getPreferredSpan(0));
                rectangle3.height = (int)view.getPreferredSpan(1);
            }
            else {
                rectangle3.width = SwingUtilities2.stringWidth(component, fontMetrics, clipString);
                leftSideBearing = SwingUtilities2.getLeftSideBearing(component, fontMetrics, clipString);
                if (leftSideBearing < 0) {
                    rectangle3.width -= leftSideBearing;
                }
                if (rectangle3.width > width) {
                    clipString = SwingUtilities2.clipString(component, fontMetrics, clipString, width);
                    rectangle3.width = SwingUtilities2.stringWidth(component, fontMetrics, clipString);
                }
                rectangle3.height = fontMetrics.getHeight();
            }
        }
        if (n3 == 1) {
            if (n4 != 0) {
                rectangle3.y = 0;
            }
            else {
                rectangle3.y = -(rectangle3.height + n9);
            }
        }
        else if (n3 == 0) {
            rectangle3.y = rectangle2.height / 2 - rectangle3.height / 2;
        }
        else if (n4 != 0) {
            rectangle3.y = rectangle2.height - rectangle3.height;
        }
        else {
            rectangle3.y = rectangle2.height + n9;
        }
        if (n4 == 2) {
            rectangle3.x = -(rectangle3.width + n9);
        }
        else if (n4 == 0) {
            rectangle3.x = rectangle2.width / 2 - rectangle3.width / 2;
        }
        else {
            rectangle3.x = rectangle2.width + n9;
        }
        int min = Math.min(rectangle2.x, rectangle3.x);
        int n11 = Math.max(rectangle2.x + rectangle2.width, rectangle3.x + rectangle3.width) - min;
        int min2 = Math.min(rectangle2.y, rectangle3.y);
        int n12 = Math.max(rectangle2.y + rectangle2.height, rectangle3.y + rectangle3.height) - min2;
        int n13;
        if (n == 1) {
            n13 = rectangle.y - min2;
        }
        else if (n == 0) {
            n13 = rectangle.y + rectangle.height / 2 - (min2 + n12 / 2);
        }
        else {
            n13 = rectangle.y + rectangle.height - (min2 + n12);
        }
        int n14;
        if (n2 == 2) {
            n14 = rectangle.x - min;
        }
        else if (n2 == 4) {
            n14 = rectangle.x + rectangle.width - (min + n11);
        }
        else {
            n14 = rectangle.x + rectangle.width / 2 - (min + n11 / 2);
        }
        rectangle3.x += n14;
        rectangle3.y += n13;
        rectangle2.x += n14;
        rectangle2.y += n13;
        if (leftSideBearing < 0) {
            rectangle3.x -= leftSideBearing;
            rectangle3.width += leftSideBearing;
        }
        if (n8 > 0) {
            rectangle3.width -= n8;
            return clipString;
        }
        return clipString;
    }
    
    public static void paintComponent(Graphics graphics, Component component, Container container, int n, int n2, int n3, int n4) {
        getCellRendererPane(component, container).paintComponent(graphics, component, container, n, n2, n3, n4, false);
    }
    
    public static void paintComponent(Graphics graphics, Component component, Container container, Rectangle rectangle) {
        paintComponent(graphics, component, container, rectangle.x, rectangle.y, rectangle.width, rectangle.height);
    }
    
    private static CellRendererPane getCellRendererPane(Component comp, Container container) {
        Container parent = comp.getParent();
        if ((parent instanceof CellRendererPane)) {
            if (parent.getParent() != container) {
                container.add(parent);
            }
        }
        else {
            parent = new CellRendererPane();
            parent.add(comp);
            container.add(parent);
        }
        return (CellRendererPane)parent;
    }
    
    public static void updateComponentTreeUI(Component component) {
        updateComponentTreeUI0(component);
        component.invalidate();
        component.validate();
        component.repaint();
    }
    
    private static void updateComponentTreeUI0(Component component) {
        if ((component instanceof JComponent)) {
            JComponent component2 = (JComponent)component;
            component2.updateUI();
            JPopupMenu componentPopupMenu = component2.getComponentPopupMenu();
            if (componentPopupMenu != null) {
                updateComponentTreeUI(componentPopupMenu);
            }
        }
        Component[] array = null;
        if ((component instanceof JMenu)) {
            array = ((JMenu)component).getMenuComponents();
        }
        else if ((component instanceof Container)) {
            array = ((Container)component).getComponents();
        }
        if (array != null) {
            for (int length = array.length, i = 0; i < length; ++i) {
                updateComponentTreeUI0(array[i]);
            }
        }
    }
    
    public static void invokeLater(Runnable runnable) {
        EventQueue.invokeLater(runnable);
    }
    
    public static void invokeAndWait(Runnable runnable) throws InterruptedException, InvocationTargetException {
        EventQueue.invokeAndWait(runnable);
    }
    
    public static boolean isEventDispatchThread() {
        return EventQueue.isDispatchThread();
    }
    
    public static int getAccessibleIndexInParent(Component component) {
        return component.getAccessibleContext().getAccessibleIndexInParent();
    }
    
    public static Accessible getAccessibleAt(Component component, Point point) {
        if ((component instanceof Container)) {
            return component.getAccessibleContext().getAccessibleComponent().getAccessibleAt(point);
        }
        if ((component instanceof Accessible)) {
            Accessible accessible = (Accessible)component;
            if (accessible != null) {
                AccessibleContext accessibleContext = accessible.getAccessibleContext();
                if (accessibleContext != null) {
                    for (int accessibleChildrenCount = accessibleContext.getAccessibleChildrenCount(), i = 0; i < accessibleChildrenCount; ++i) {
                        Accessible accessibleChild = accessibleContext.getAccessibleChild(i);
                        if (accessibleChild != null) {
                            accessibleContext = accessibleChild.getAccessibleContext();
                            if (accessibleContext != null) {
                                AccessibleComponent accessibleComponent = accessibleContext.getAccessibleComponent();
                                if (accessibleComponent != null && accessibleComponent.isShowing()) {
                                    Point location = accessibleComponent.getLocation();
                                    if (accessibleComponent.contains(new Point(point.x - location.x, point.y - location.y))) {
                                        return accessibleChild;
                                    }
                                }
                            }
                        }
                    }
                }
            }
            return (Accessible)component;
        }
        else {
            return null;
        }
    }
    
    public static AccessibleStateSet getAccessibleStateSet(Component component) {
        return component.getAccessibleContext().getAccessibleStateSet();
    }
    
    public static int getAccessibleChildrenCount(Component component) {
        return component.getAccessibleContext().getAccessibleChildrenCount();
    }
    
    public static Accessible getAccessibleChild(Component component, int n) {
        return component.getAccessibleContext().getAccessibleChild(n);
    }
    
    @Deprecated
    public static Component findFocusOwner(Component component) {
        Component focusOwner = KeyboardFocusManager.getCurrentKeyboardFocusManager().getFocusOwner();
        while (focusOwner != null) {
            if (focusOwner == component) {
                return focusOwner;
            }
            focusOwner = ((focusOwner instanceof Window) ? null : focusOwner.getParent());
        }
        return null;
    }
    
    public static JRootPane getRootPane(Component parent) {
        if ((parent instanceof RootPaneContainer)) {
            return ((RootPaneContainer)parent).getRootPane();
        }
        while (parent != null) {
            if ((parent instanceof JRootPane)) {
                return (JRootPane)parent;
            }
            parent = parent.getParent();
        }
        return null;
    }
    
    public static Component getRoot(Component parent) {
        Component component = null;
        while (parent != null) {
            if ((parent instanceof Window)) {
                return parent;
            }
            if ((parent instanceof Applet)) {
                component = parent;
            }
            parent = parent.getParent();
        }
        return component;
    }
    
    static JComponent getPaintingOrigin(JComponent parent) {
        while ((parent = parent.getParent()) instanceof JComponent) {
            JComponent component = (JComponent)parent;
            if (component.isPaintingOrigin()) {
                return component;
            }
            continue;
        }
        return null;
    }
    
    public static boolean processKeyBindings(KeyEvent keyEvent) {
        if (keyEvent != null) {
            if (keyEvent.isConsumed()) {
                return false;
            }
            Component component = keyEvent.getComponent();
            int n = keyEvent.getID() == 401;
            if (!isValidKeyEventForKeyBindings(keyEvent)) {
                return false;
            }
            while (component != null) {
                if ((component instanceof JComponent)) {
                    return ((JComponent)component).processKeyBindings(keyEvent, (boolean)n);
                }
                if (component instanceof Applet || component instanceof Window) {
                    return JComponent.processKeyBindingsForAllComponents(keyEvent, (Container)component, (boolean)n);
                }
                component = component.getParent();
            }
        }
        return false;
    }
    
    static boolean isValidKeyEventForKeyBindings(KeyEvent keyEvent) {
        return true;
    }
    
    public static boolean notifyAction(Action action, KeyStroke keyStroke, KeyEvent keyEvent, Object source, int modifiers) {
        if (action == null) {
            return false;
        }
        if ((action instanceof UIAction)) {
            if (!((UIAction)action).isEnabled(source)) {
                return false;
            }
        }
        else if (!action.isEnabled()) {
            return false;
        }
        Object value = action.getValue("ActionCommandKey");
        int n = value == null && action instanceof JComponent.ActionStandin;
        String command;
        if (value != null) {
            command = value.toString();
        }
        else if (n == 0 && keyEvent.getKeyChar() != '\uffff') {
            command = String.valueOf(keyEvent.getKeyChar());
        }
        else {
            command = null;
        }
        action.actionPerformed(new ActionEvent(source, 1001, command, keyEvent.getWhen(), modifiers));
        return true;
    }
    
    public static void replaceUIInputMap(JComponent component, int n, InputMap parent) {
        InputMap inputMap = component.getInputMap(n, parent != null);
        while (inputMap != null) {
            InputMap parent2 = inputMap.getParent();
            if (parent2 == null || parent2 instanceof UIResource) {
                inputMap.setParent(parent);
                return;
            }
            inputMap = parent2;
        }
    }
    
    public static void replaceUIActionMap(JComponent component, ActionMap parent) {
        ActionMap actionMap = component.getActionMap(parent != null);
        while (actionMap != null) {
            ActionMap parent2 = actionMap.getParent();
            if (parent2 == null || parent2 instanceof UIResource) {
                actionMap.setParent(parent);
                return;
            }
            actionMap = parent2;
        }
    }
    
    public static InputMap getUIInputMap(JComponent component, int n) {
        InputMap inputMap = component.getInputMap(n, false);
        while (inputMap != null) {
            InputMap parent = inputMap.getParent();
            if ((parent instanceof UIResource)) {
                return parent;
            }
            inputMap = parent;
        }
        return null;
    }
    
    public static ActionMap getUIActionMap(JComponent component) {
        ActionMap actionMap = component.getActionMap(false);
        while (actionMap != null) {
            ActionMap parent = actionMap.getParent();
            if ((parent instanceof UIResource)) {
                return parent;
            }
            actionMap = parent;
        }
        return null;
    }
    
    static Frame getSharedOwnerFrame() throws HeadlessException {
        Frame frame = (Frame)appContextGet(SwingUtilities.sharedOwnerFrameKey);
        if (frame == null) {
            frame = new SwingUtilities.SharedOwnerFrame();
            appContextPut(SwingUtilities.sharedOwnerFrameKey, frame);
            return frame;
        }
        return frame;
    }
    
    static WindowListener getSharedOwnerFrameShutdownListener() throws HeadlessException {
        return (WindowListener)getSharedOwnerFrame();
    }
    
    static Object appContextGet(Object key) {
        return AppContext.getAppContext().get(key);
    }
    
    static void appContextPut(Object key, Object value) {
        AppContext.getAppContext().put(key, value);
    }
    
    static void appContextRemove(Object key) {
        AppContext.getAppContext().remove(key);
    }
    
    static Class<?> loadSystemClass(String s) throws ClassNotFoundException {
        ReflectUtil.checkPackageAccess(s);
        return Class.forName(s, true, Thread.currentThread().getContextClassLoader());
    }
    
    static boolean isLeftToRight(Component component) {
        return component.getComponentOrientation().isLeftToRight();
    }
    
    private SwingUtilities() {
        throw new Error("SwingUtilities is just a container for static methods");
    }
    
    static boolean doesIconReferenceImage(Icon icon, Image image) {
        return ((icon != null && icon instanceof ImageIcon) ? ((ImageIcon)icon).getImage() : null) == image;
    }
    
    static int findDisplayedMnemonicIndex(String s, int n) {
        if (s == null || n == 0) {
            return -1;
        }
        int upperCase = Character.toUpperCase((char)n);
        int lowerCase = Character.toLowerCase((char)n);
        int index = s.indexOf(upperCase);
        int index2 = s.indexOf(lowerCase);
        if (index == -1) {
            return index2;
        }
        if (index2 == -1) {
            return index;
        }
        return (index2 < index) ? index2 : index;
    }
    
    public static Rectangle calculateInnerArea(JComponent component, Rectangle rectangle) {
        if (component == null) {
            return null;
        }
        Insets insets = component.getInsets();
        if (rectangle == null) {
            rectangle = new Rectangle();
        }
        rectangle.x = insets.left;
        rectangle.y = insets.top;
        rectangle.width = component.getWidth() - insets.left - insets.right;
        rectangle.height = component.getHeight() - insets.top - insets.bottom;
        return rectangle;
    }
    
    static void updateRendererOrEditorUI(Object o) {
        if (o == null) {
            return;
        }
        Component component = null;
        if ((o instanceof Component)) {
            component = (Component)o;
        }
        if ((o instanceof DefaultCellEditor)) {
            component = ((DefaultCellEditor)o).getComponent();
        }
        if (component != null) {
            updateComponentTreeUI(component);
        }
        return;
    }
    
    public static Container getUnwrappedParent(Component component) {
        Container container;
        for (container = component.getParent(); container instanceof JLayer; container = container.getParent()) {}
        return container;
    }
    
    public static Component getUnwrappedView(JViewport viewport) {
        Component component;
        for (component = viewport.getView(); component instanceof JLayer; component = ((JLayer)component).getView()) {}
        return component;
    }
    
    static Container getValidateRoot(Container container, boolean b) {
        Container container2 = null;
        while (container != null) {
            if (!container.isDisplayable() || container instanceof CellRendererPane) {
                return null;
            }
            if (container.isValidateRoot()) {
                container2 = container;
                break;
            }
            container = container.getParent();
        }
        if (container2 == null) {
            return null;
        }
        while (container != null) {
            if (!container.isDisplayable() || (b && !container.isVisible())) {
                return null;
            }
            if (container instanceof Window || container instanceof Applet) {
                return container2;
            }
            container = container.getParent();
        }
        return null;
    }
    
    static {
        SwingUtilities.canAccessEventQueue = false;
        SwingUtilities.eventQueueTested = false;
        sharedOwnerFrameKey = new StringBuffer("SwingUtilities.sharedOwnerFrame");
    }
    
    static class SharedOwnerFrame extends Frame implements WindowListener
    {
        @Override
        public void addNotify() {
            super.addNotify();
            this.installListeners();
        }
        
        void installListeners() {
            Window[] ownedWindows;
            for (Window window : ownedWindows = this.getOwnedWindows()) {
                if (window != null) {
                    window.removeWindowListener(this);
                    window.addWindowListener(this);
                }
            }
        }
        
        @Override
        public void windowClosed(WindowEvent windowEvent) {
            synchronized (this.getTreeLock()) {
                Window[] ownedWindows;
                for (Window window : ownedWindows = this.getOwnedWindows()) {
                    if (window != null) {
                        if (window.isDisplayable()) {
                            return;
                        }
                        window.removeWindowListener(this);
                    }
                }
                this.dispose();
            }
        }
        
        @Override
        public void windowOpened(WindowEvent windowEvent) {
        }
        
        @Override
        public void windowClosing(WindowEvent windowEvent) {
        }
        
        @Override
        public void windowIconified(WindowEvent windowEvent) {
        }
        
        @Override
        public void windowDeiconified(WindowEvent windowEvent) {
        }
        
        @Override
        public void windowActivated(WindowEvent windowEvent) {
        }
        
        @Override
        public void windowDeactivated(WindowEvent windowEvent) {
        }
        
        @Override
        public void show() {
        }
        
        @Override
        public void dispose() {
            try {
                this.getToolkit().getSystemEventQueue();
                super.dispose();
            }
            catch (Exception ex) {}
        }
    }
}
